/* $ Id: $
 * User: kortenka
 * Date: 29.10.2002
 * Time: 14:51:54
 */
package eaa.sorting.intern;

import eaa.sorting.Comparable;

public class Array {

    private Element[] elements;
    private String info;

    private static int swaps = 0;
    private static int copies = 0;
    private static int storage = 0;


    private Array() {

    }

    /**
     * liefert ein richtig sortiertes Array
     *
     * @param size die Groesse des Array
     * @return ein Array von vergleichbaren Elementen
     *
     * @see eaa.sorting.Comparable
     */
    public static Array getSortedArray(int size) {
        storage += size;
        Array a = new Array();
        a.elements = new Element[size];
        for (int i = 0; i < size; i++) {
            a.elements[i] = new Element(i);
        }
        a.info = "vorsortiertes Array";
        return a;
    }

    /**
     * liefert ein Array, welches rueckwaerts sortiert ist
     * @param size die Groesse des Array
     * @return ein Array von vergleichbaren Elementen
     *
     * @see eaa.sorting.Comparable
     */
    public static Array getReverseSortedArray(int size) {
        storage += size;
        Array a = new Array();
        a.elements = new Element[size];
        for (int i = 0; i < size; i++) {
            a.elements[i] = new Element(size - i);
        }
        a.info = "rueckwaerts sortiertes Array";
        return a;
    }

    /**
     * liefert ein mehr oder weniger zufaellig gefuelltes Array.
     *
     * @param size die Groesse des Array
     * @return ein Array von vergleichbaren Elementen
     *
     * @see eaa.sorting.Comparable
     */
    public static Array getRandomArray(int size) {
        storage += size;
        Array a = new Array();
        a.elements = new Element[size];
        for (int i = 0; i < size; i++) {
            a.elements[i] = new Element((int) (Math.random() * size));
        }
        a.info = "zufaelliges Array";
        return a;
    }

    /**
     * liefert ein leeres Array zum Aufnehmen von Werten
     * @param size die Groesse des Arrays
     * @return ein Array
     */
    public static Array getStorage(int size) {
        storage += size;
        Array a = new Array();
        a.elements = new Element[size];
        return a;
    }

    /**
     * Vertauscht die Elemente an Stelle i und j.
     *
     * @param i 0 &lt;= i &lt; Groesse des Arrays
     * @param j 0 &lt;= i &lt; Groesse des Arrays
     */
    public void swap(int i, int j) {
        swaps++;
        Element h = elements[i];
        elements[i] = elements[j];
        elements[j] = h;
    }

    /**
     * Kopiert ein Element von einem Array ins andere
     * @param from der Index des Elements in diesem Array
     * @param array das andere Array
     * @param to der Index des Elements in dem neuen Array
     */
    public void copyTo(int from, Array array, int to) {
        copies++;
        array.elements[to] = elements[from];
    }

    /**
     * Testet, ob ein Array sortiert ist.
     * @return
     */
    public boolean isSorted() {
        for (int i = 1; i < elements.length; i++) {
            if (elements[i - 1].isGreater(elements[i])) {
                return false;
            }
        }
        return true;
    }

    /**
     * Liefert die Anzahl der Vertauschungen seit dem letzten
     * reset.
     * @return ein int >= 0
     */

    public static int getSwapCount() {
        return swaps;
    }

    /**
     * Liefert die Anzahl der Kopieroperationen seit dem letzten
     * reset.
     * @return ein int >= 0
     */

    public static int getCopyCount() {
        return copies;
    }

    /**
     * Liefert den Speicherbedarf seit dem letzten
     * reset.
     * @return ein int >= 0
     */

    public static int getStorageUsage() {
        return storage;
    }

    /**
     * Setzt den Vertauschungszaehler zurueck auf 0.
     */
    static void resetCounter() {
        swaps = 0;
        copies = 0;
        storage = 0;
    }

    /**
     * Liefert die Groesse des Arrays
     * @return die Anzahl der Elemente des Arrays
     */
    public int size() {
        return elements.length;
    }

    /**
     * liefert den Schluessel des Arrays an Stelle i
     * @param i
     * @return
     */
    public Comparable elementAt(int i) {
        return elements[i];
    }


    public String toString() {
        StringBuffer sb = new StringBuffer(elements.length * 5);
        sb.append(elements[0]);
        for (int i = 1; i < elements.length; i++) {
            Element element = elements[i];
            sb.append(",");
            sb.append(element.toString());
        }
        return sb.toString();
    }

    public static String stats() {
        return "Storage: "+getStorageUsage() + "  Swaps: "+getSwapCount()+ "  Assigns: "+ getCopyCount();
    }

    /**
     * Erzeugt ein identisches Array
     * @return ein Array, welches die gleichen Elemente enthaelt.
     */
    protected Array getCopy() {
        storage += size();
        Array a = new Array();
        a.elements = new Element[size()];
        System.arraycopy(this.elements, 0, a.elements, 0, size());
        a.info = info;
        return a;
    }

    public static int getCost() {
        return storage + copies + swaps;
    }

    public String getInfo() {
        return info + " ["+size()+"]";
    }
}









